﻿#ifndef BURNDATA_H
#define BURNDATA_H

#include "src/model/logicburndata.h"
#include "src/view/multiselectionfiledialog.h"
#include "src/process/processopenfile.h"
#include "inputchange.h"
#include "tipschange.h"
#include <QWidget>
#include <QMutex>
#include <QAction>
#include <QScopedPointer>
#include <klineframe.h>
#include <kbreadcrumb.h>
#include <ktoolbutton.h>

class MyMenu;
class LogicBurnData;
namespace Ui {
class BurnData;
}

class MyTreeView : public QTreeView
{
    Q_OBJECT
public:
    explicit MyTreeView(QWidget *parent = nullptr);

protected:
    void keyPressEvent(QKeyEvent* event) Q_DECL_OVERRIDE;
    void mousePressEvent(QMouseEvent* event) Q_DECL_OVERRIDE;

};

class BurnData : public QWidget
{
    Q_OBJECT
public:
    static BurnData *instance();
public:
    BurnData(const BurnData*)=delete;
    BurnData *operator =(const BurnData*)=delete;
public:
    void refresh();
    static bool m_bDirReNameing; // 是否在重命名中
public:
    void updatePosition(QPoint);
    void clearHistory() {
        m_back_list.clear();
        m_forward_list.clear();
    }

    bool canGoBack();
    bool canGoForward();
    void delModelIndex(QModelIndex);

protected:
    bool event(QEvent *event) Q_DECL_OVERRIDE;
    bool eventFilter(QObject *watched, QEvent *event) Q_DECL_OVERRIDE;
    void paintEvent(QPaintEvent *event) Q_DECL_OVERRIDE;
    void resizeEvent(QResizeEvent *event) Q_DECL_OVERRIDE;
    void keyPressEvent(QKeyEvent* event) Q_DECL_OVERRIDE;
private:
    explicit BurnData(QWidget *parent = nullptr);
    ~BurnData();
private:
    void initAction();
    void linkEntry(QModelIndex);
    void dataViewChanged(QModelIndex& idx);
    void autoResetCrumb();
    void updateDisplayList(int count);
    void clearCrumb();
    void resetText(QPushButton* item);
    bool m_isAutoReset = false;
    int m_offsetRemovedTabCount = 0;


public slots:
    void slotChangeUIFontSize(int fontSize);
    void slotChangeUIFontname(QString fontName);
    void slotStopLoadingData();

private slots:
    void slotSetDefaultAddWindowVisible(bool isVisible);
    void slotsEntry(QModelIndex);
    void slotsSize(unsigned long);
    void slotsSelectionChanged();
    void slotsAddFiles(bool);
    void slotsDeleteFiles(bool);
    void slotsCleanFiles(bool);
    void slotsNewFolder(bool);
    void slotsFilterHidden(bool);
    void slotsFilterBroken(bool);
    void slotsRepleaceSymbolLinks(bool);
    void slotsParent();
    void slotsOpen();
    void slotsOpenLocal();
    void slotsRename();
    void slotsProperties();
    void slotsTreeMenuByRightKey(const QPoint &pos);
    void slotsTreeMenuByShortcutKey();
    void slotsDeleteFilesByKeyPress(bool);

    void goBack();
    void goForward();
    void directoryChangeSlot(int);
    void slotsBurnerIndexChanged();

private:
    static BurnData          *m_oInstance;
    static QMutex             m_oMutex;
    bool                      m_bIsWayland;
    QLabel *title;
    kdk::KHLineFrame *bottomLine;
private:
    LogicBurnData            *m_oModel;
    MyMenu                   *m_oMenu;
    QAction                  *m_oOpen;
    QAction                  *m_oOpenLocal;
    QAction                  *m_oRename;
    QAction                  *m_oDelete;
    QAction                  *m_oNewFolder;
    QAction                  *m_oProperties;
    ProcessOpenFile          *m_oOpenFile;
    QModelIndex               m_oAddIndex;

    kdk::KBreadCrumb         *mDirectoryCrumb;
    kdk::KToolButton *m_go_back;
    kdk::KToolButton *m_go_forward;
    QMap<int, QModelIndex> *mDirectoryMap;   //目录映射，便于快速切换目录与QTreeView显示内容
                                                 //TODO: const QModelIndex传递为什么有问题?
    QModelIndexList m_back_list;
    QModelIndexList m_forward_list;
    QPoint                    m_oPosition;
    QModelIndex               m_rootIndex;
    QStringList m_textList;
private:
    Ui::BurnData *ui;
Q_SIGNALS:
    void changeCurrentIndex(int);
    void burnDataToDiscEnabled(bool);
    void updateDisk(int, void *);
    void newNode(void *);
    void delData(int);
    void viewFileterFile(bool);
};
#define PAGE_BurnData BurnData::instance()
#endif // BURNDATA_H
