#ifndef BUSNESSSETTING_H
#define BUSNESSSETTING_H

#include <QWidget>
#include <QDebug>
#include <QLabel>
#include <QLineEdit>
#include <QEvent>

namespace Ui {
class BusnessSetting;
}

class CustomLineEdit : public QLineEdit{
    Q_OBJECT
public:
    explicit CustomLineEdit(QWidget *parent = nullptr);
protected:
    void changeEvent(QEvent* event) override{
        if(event->type() == QEvent::EnabledChange || event->type() == QEvent::FocusIn){
            if(isReadOnly()){
                setCursor(Qt::IBeamCursor);
            }else{
                setCursor(Qt::IBeamCursor);
            }
        }
        QLineEdit::changeEvent(event);
    }
};

class DataSetting;
class BusnessSetting : public QWidget
{
    Q_OBJECT

public:
    explicit BusnessSetting(QWidget *parent = nullptr);
    ~BusnessSetting();

protected:
    void paintEvent(QPaintEvent *event) Q_DECL_OVERRIDE;
    void checkVolumnID();
    void checkCopyright();
    void checkTempPath();
    void updateModelDialogEnsureBtnState();

public slots:
    void load(void *);
    void setSpeed(int i) { m_iSpeed = i; }
    void setBurnFormat(QString s) { m_SBurnFormat = s; }
    void setClean(bool b) { m_bClean = b; }
    void setCreateISO(bool b) { m_bCreateISO = b; }
    void setCloseSession(bool b) { m_bCloseSession = b; }
    void setVerficationData(bool b) { m_bVerfication = b; }
    void setOpen(bool b) { m_bOpen = b; }
    void setPreparer(QString s) { m_oPreparer = s; }
    void setPublisher(QString s) { m_oPublisher = s; }
    void setVolumnID(QString s);
    void setCopyright(QString s) { m_oCopyright = s; }
    void setTempPath(QString s);
    void setIsShowBeforBurn(bool b) { m_isShowBeforBurn = b; }

    void resetText(QLabel* item);
    void updateLabelText();
    void slotChangeUIFontSize(int fontSize);


private:
    void setTemporaryPath(QString);
    void uiAdjust();
    void initTempPathLineEdit();
    void slotChangeFileSystemFormat(int);

private slots:
    void slotsBrowse();
    void slotsAccept();

private:
    int         m_iSpeed;
    bool        m_bClean;
    bool        m_bCreateISO;
    bool        m_bCloseSession;
    bool        m_bVerfication;
    bool        m_bOpen;
    bool        m_isShowBeforBurn;
    bool        m_isVaildVolumID;
    bool        m_isVaildCopyright;
    bool        m_isVaildTempPath;
    void       *m_oCDROM;
    QString     m_oPreparer;
    QString     m_oPublisher;
    QString     m_oVolumnID;
    QString     m_oCopyright;
    QString     m_oTemporaryPth;
    QString     m_oBrowsePath;
    QStringList m_oSpeed;
    QList<int>  m_oSpeeds;
    QString     m_SBurnFormat;
    QList<QString>  m_oBurnFormat;
private:
    Ui::BusnessSetting *ui;
Q_SIGNALS:
    void ensureBtnStateChanged(bool);
    void cancelBtnStateChanged(bool);
    void title(QString);
};

#endif // BUSNESSSETTING_H
